@@ -1,11 +1,12 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: apache.tcl,v 1.8 1999/07/20 19:05:55 jfontain Exp $}
+set rcsId {$Id: apache.tcl,v 2.8 2001/01/27 15:08:19 jfontain Exp $}
 
-package provide apache 1.2
-package require apacheutilities 1.1
 
+package provide apache [lindex {$Revision: 2.8 $} 1]
+package require miscellaneous 1
+package require apacheutilities 1
 
 namespace eval apache {
 
@@ -25,13 +26,10 @@ namespace eval apache {
         10,label accesses/second 10,type real 10,message {current number of accesses per second}
         11,label kilobytes/second 11,type real 11,message {current kilobytes served per second}
         pollTimes {10 5 20 30 60 120 300}
-        views {
-            {visibleColumns {3 4 8 9} sort {3 increasing}}
-            {visibleColumns {1 2 5 6 7 10 11} sort {1 increasing}}
-        }
-        switches {-r 1 --remote 1}
+        views {{visibleColumns {1 2 3 4 5 6 7 8 9 10 11} swap 1}}
+        switches {-r 1 --remote 1 --proxyhost 1 --proxyport 1}
     }
-    set file [open [file join $::packageDirectory(apache) apache.htm]]
+    set file [open apache.htm]
     set data(helpText) [read $file]                                                           ;# initialize HTML help data from file
     close $file
 
@@ -39,34 +37,53 @@ namespace eval apache {
         upvar $optionsName options
         variable url
         variable data
+        variable host
 
         set url [url options host apache ?auto]
         set data(identifier) apache($host)
+        configure options
+        ::http::geturl $url -validate 1                                    ;# test server so an error is generated if not responding
     }
 
     proc update {} {
         variable url
         variable transaction
+        variable data
 
         if {[info exists transaction]} return                                                                                ;# busy
-        set transaction [::http::geturl $url -command ::apache::completed]
+        if {[catch {set transaction [::http::geturl $url -command ::apache::completed]} message]} {
+            flashMessage "apache error: $message"
+            resetData
+            incr data(updates)                                                                                   ;# show void values
+        }
     }
 
     proc completed {token} {
         variable data
         variable last
         variable transaction
+        variable host
 
-        set valid [expr {[lindex [set ${token}(http)] 1]==200}]                               ;# sole HTTP code for good transaction
+        set valid [expr {[::http::ncode $token]==200}]                                        ;# sole HTTP code for good transaction
         if {$valid} {
             foreach line [split [set ${token}(body)] \n] {
                 if {![regexp {^(.+): (.+)$} $line dummy name value]} break                             ;# stop at first invalid line
                 set current($name) $value
             }
+            set size [array size current]
+            if {$size==3} {                                                  ;# only returned: busy and idle servers plus scoreboard
+                flashMessage "apache error: extended status not enabled on $host"
+            }
             # check that all statistics are present (per second values are not always returned when server was just (re)started)
-            set valid [expr {[array size current]==10}]
+            set valid [expr {$size==10}]
+        } else {
+            if {[::http::ncode $token]==404} {
+                flashMessage "apache error: no server status on $host"
+            } else {
+                flashMessage "apache error: [::http::code $token]"
+            }
         }
-        unset $token                                                     ;# free transaction data array (fully qualified and global)
+        ::http::cleanup $token                                                                              ;# free transaction data
         if {$valid} {
             set data(0,1) $current(Total Accesses)
             set data(0,2) $current(Total kBytes)
@@ -85,12 +102,19 @@ namespace eval apache {
                 array set data {0,10 0 0,11 0}
             }
             array set last [list accesses $current(Total Accesses) kilobytes $current(Total kBytes) uptime $current(Uptime)]
-        } else {                                                                                            ;# an HTTP error occured
-            array set data {0,1 0 0,2 0 0,3 0 0,4 ? 0,5 0 0,6 0 0,7 0 0,8 0 0,9 0 0,10 0 0,11 0}          ;# show meaningless values
-            catch {unset last}
+        } else {                                                                                             ;# a HTTP error occured
+            resetData
         }
         unset transaction                                                                              ;# ready for next transaction
         incr data(updates)
     }
 
+    proc resetData {} {
+        variable data
+        variable last
+
+        array set data {0,1 ? 0,2 ? 0,3 ? 0,4 ? 0,5 ? 0,6 ? 0,7 ? 0,8 ? 0,9 ? 0,10 ? 0,11 ?}                     ;# show void values
+        catch {unset last}
+    }
+
 }
