@@ -1,7 +1,7 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: canvhand.tcl,v 2.2 1999/08/28 10:40:20 jfontain Exp $}
+set rcsId {$Id: canvhand.tcl,v 2.12 2000/12/31 23:14:19 jfontain Exp $}
 
 class canvasWindowManager {
 
@@ -24,20 +24,17 @@ class canvasWindowManager {
 
         proc options {this} {
             return [list\
-                [list\
-                    -background background Background\
-                    $widget::(default,ButtonBackgroundColor) $widget::(default,ButtonBackgroundColor)\
-                ]\
-                [list -borderwidth borderWidth BorderWidth 3]\
-                [list -handlesize handleSize HandleSize 7 7]\
-                [list -path path Path {} {}]\
-                [list -relief relief Relief ridge]\
-                [list -setheight setHeight SetHeight {} {}]\
-                [list -setwidth setWidth SetWidth {} {}]\
-                [list -setx setX SetX 0 0]\
-                [list -sety setY SetY 0 0]\
-                [list -static static Static 0]\
-                [list -title title Title {} {}]\
+                [list -background $widget::option(button,background) $widget::option(button,background)]\
+                [list -borderwidth 3]\
+                [list -handlesize 7 7]\
+                [list -path {} {}]\
+                [list -relief ridge]\
+                [list -setheight {} {}]\
+                [list -setwidth {} {}]\
+                [list -setx 0 0]\
+                [list -sety 0 0]\
+                [list -static 0]\
+                [list -title {} {}]\
             ]
         }
 
@@ -58,8 +55,19 @@ class canvasWindowManager {
             set ($this,filled) 1
         }
 
-        foreach option {-background -relief -borderwidth} {
-            proc set$option {this value} "\$widget::(\$this,path) configure $option \$value"
+        proc set-background {this value} {
+            $widget::($this,path) configure -background $value
+        }
+
+        proc set-borderwidth {this value} {
+            if {$value<3} {
+                set value 3                                                        ;# so that borders are large enough to be grabbed
+            }
+            $widget::($this,path) configure -borderwidth $value
+        }
+
+        proc set-relief {this value} {
+            $widget::($this,path) configure -relief $value
         }
 
         proc set-setheight {this value} {
@@ -105,7 +113,7 @@ class canvasWindowManager {
                 bind $path <Button1-Motion> {}
                 bind $path <ButtonPress-1> {}
                 bind $path <ButtonRelease-1> "canvasWindowManager::handles::toggleVisibility $this"
-                $path configure -cursor arrow        ;# neutral cursor for user feedback since visibility toggling action is allowed
+                $path configure -cursor left_ptr
             } else {
                 bind $path <Configure> "canvasWindowManager::handles::resize $this %w %h"                    ;# monitor size changes
                 bind $path <Motion> "canvasWindowManager::handles::setCursor $this %x %y"
@@ -115,30 +123,35 @@ class canvasWindowManager {
                 bind $path <ButtonPress-1> "canvasWindowManager::handles::buttonPress $this %x %y"
                 bind $path <ButtonRelease-1> "canvasWindowManager::handles::buttonRelease $this"
             }
+            bind $path <ButtonRelease-2> "canvasWindowManager::handles::toggleVisibility $this"
             if {[info exists ($this,label)]} {                                                                         ;# title area
                 set path $($this,label)
                 if {$static} {
-                    $path configure -cursor arrow
+                    $path configure -cursor left_ptr
                     bind $path <Button1-Motion> {}
                     bind $path <ButtonPress-1> {}
                     bind $path <ButtonRelease-1> "canvasWindowManager::handles::toggleVisibility $this"
                 } else {                                                                      ;# allow moving window from title area
                     $path configure -cursor fleur
                     bind $path <Button1-Motion> "canvasWindowManager::handles::buttonMotion $this %x %y"
-                    bind $path <ButtonPress-1> "canvasWindowManager::handles::buttonPress $this %x %y"
+                    bind $path <ButtonPress-1> "
+                        set canvasWindowManager::handles::($this,direction) {}           ;# moving, not resizing, so reset direction
+                        canvasWindowManager::handles::buttonPress $this %x %y
+                    "
                     bind $path <ButtonRelease-1> "canvasWindowManager::handles::buttonRelease $this"
                 }
+                bind $path <ButtonRelease-2> "canvasWindowManager::handles::toggleVisibility $this"
             }
         }
 
         proc buttonMotion {this x y} {
-            set ::canvasWindowManager::handles::(motion) {}
+            set (motion) {}
             updateOutline $this $x $y
         }
 
         proc buttonPress {this x y} {
-            set ::canvasWindowManager::handles::(xLast) $x
-            set ::canvasWindowManager::handles::(yLast) $y
+            set (xLast) $x
+            set (yLast) $y
             lifoLabel::push $global::messenger {}    ;# in case no other string is pushed before button release event pops messenger
             createOutline $this
         }
@@ -156,13 +169,12 @@ class canvasWindowManager {
             if {[info exists (motion)]} {                                                              ;# moving or resizing occured
                 updateGeometry $this
                 stack $this raise                                                   ;# always place widget on top after acting on it
-                unset ::canvasWindowManager::handles::(motion)
+                unset (motion)
             } else {                                                                                ;# no moving or resizing occured
                 toggleVisibility $this
             }
             destroyOutline $this
-            unset ::canvasWindowManager::handles::(xLast) ::canvasWindowManager::handles::(yLast)\
-                ::canvasWindowManager::handles::(hidden)
+            unset (xLast) (yLast) (hidden)
         }
 
         proc resize {this width height} {
@@ -265,11 +277,11 @@ class canvasWindowManager {
             if {($yFrame+$y)<0} {
                 set y [expr {-$yFrame}]                                             ;# use expr to properly handle consecutive signs
             }
-            set width [winfo width $canvas]
+            set width [maximum [$canvas cget -width] [winfo width $canvas]]            ;# width can be extended using window manager
             if {($xFrame+$x)>=$width} {
                 set x [expr {$width-$xFrame-1}]
             }
-            set height [winfo height $canvas]
+            set height [maximum [$canvas cget -height] [winfo height $canvas]]        ;# height can be extended using window manager
             if {($yFrame+$y)>=$height} {
                 set y [expr {$height-$yFrame-1}]
             }
@@ -277,8 +289,8 @@ class canvasWindowManager {
             if {[string length $($this,direction)]==0} {                                                     ;# moving, not resizing
                 $canvas move outline [expr {$x-$(xLast)}] [expr {$y-$(yLast)}]
                 lifoLabel::push $global::messenger [$canvas coords outline]               ;# display new coordinates in message area
-                set ::canvasWindowManager::handles::(xLast) $x
-                set ::canvasWindowManager::handles::(yLast) $y
+                set (xLast) $x
+                set (yLast) $y
                 return
             }
 
@@ -331,7 +343,7 @@ class canvasWindowManager {
             foreach side {top bottom left right} {
                 $order [$canvas itemcget $($side,item) -window]
             }
-            set ::canvasWindowManager::handles::(hidden) [string compare $order raise]
+            set (hidden) [string compare $order raise]
         }
 
         proc displayOutline {this x y width height} {                                          ;# coordinates are relative to canvas
