@@ -1,20 +1,19 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: diskstats.tcl,v 2.1 1999/08/27 20:19:00 jfontain Exp $}
+set rcsId {$Id: diskstats.tcl,v 2.10 2001/01/27 15:08:19 jfontain Exp $}
 
-package provide diskstats 1.1
-package require filesystem
+
+package provide diskstats [lindex {$Revision: 2.10 $} 1]
 
 namespace eval diskstats {
 
     set file [open /proc/version]                                                                      ;# first check kernel version
-    regexp {^[0-9\.]+} [lindex [gets $file] 2] version                             ;# ignore extra characters, such as in 2.2.0-pre1
+    regexp {^[\d\.]+} [lindex [gets $file] 2] version                             ;# ignore extra characters, such as in 2.2.0-pre1
     close $file
 
     if {[package vcompare $version 2]<0} {
-        puts stderr {diskstats module needs at least a version 2 kernel}
-        exit 1
+        error {at least a version 2 kernel is needed}
     }
     set full [expr {[package vcompare $version 2.1]>=0}]                                 ;# 2.1 kernels and up support full features
 
@@ -28,7 +27,7 @@ namespace eval diskstats {
         pollTimes {10 5 20 30 60 120 300}
         sort {0 increasing}
     }
-    set file [open [file join $::packageDirectory(diskstats) diskstats.htm]]
+    set file [open diskstats.htm]
     set data(helpText) [read $file]
     close $file
 
@@ -36,7 +35,7 @@ namespace eval diskstats {
         array set data {
             5,label device 5,type ascii 5,message {device (sd*: SCSI, hd*: IDE)}
             6,label type 6,type ascii 6,message {type or media}
-            7,label size 7,type integer 7,message {size in KiloBytes}
+            7,label size 7,type real 7,message {size in MegaBytes}
             8,label model 8,type ascii 8,message {model description} 8,anchor left
             views {
                 {visibleColumns {0 1 2 3 4} sort {0 increasing}}
@@ -82,7 +81,8 @@ namespace eval diskstats {
                 if {$scsiFromCharacter($type)||([string length $data($index,5)]==0)} {
                     set data($index,5) $device
                     scan $line {%*u %*u %u} blocks
-                    set data($index,7) [expr {round($blocks*([lindex [filesystem::statistics /dev/$device] 0]/1024.0))}]
+                    # assume block size is always 1024 in /proc/partitions and avoid integer overflowing
+                    set data($index,7) [expr {round($blocks/1024.0)}]
                     if {$scsiFromCharacter($type)} {                                                                         ;# SCSI
                         set file [open /proc/scsi/scsi]
                         gets $file                                                                               ;# skip header line
@@ -118,11 +118,29 @@ namespace eval diskstats {
         }
 
         seek $statistics 0                                                                   ;# always rewind before retrieving data
-        while {![string match disk* [gets $statistics]]} {}     ;# forward to first disk entry line (in case there are several CPUs)
-        scan [gets $statistics] {disk_rio %u %u %u %u} data(0,1) data(1,1) data(2,1) data(3,1)
-        scan [gets $statistics] {disk_wio %u %u %u %u} data(0,2) data(1,2) data(2,2) data(3,2)
-        scan [gets $statistics] {disk_rblk %u %u %u %u} data(0,3) data(1,3) data(2,3) data(3,3)
-        scan [gets $statistics] {disk_wblk %u %u %u %u} data(0,4) data(1,4) data(2,4) data(3,4)
+        # forward to first disk entry line (in case there are several CPUs)
+        while {![string match disk* [set line [gets $statistics]]]} {}
+        if {[string match disk_io* $line]} {                                                                           ;# 2.4 kernel
+            scan $line {disk_io: (%*u,%u):(%*u,%u,%u,%u,%u)} index readIOs readBlocks writeIOs writeBlocks
+            set found($index) {}
+            array set data [list $index,1 $readIOs $index,2 $writeIOs $index,3 $readBlocks $index,4 $writeBlocks]
+            while {\
+                [scan [gets $statistics] {disk_io: (%*u,%u):(%*u,%u,%u,%u,%u)} index readIOs readBlocks writeIOs writeBlocks]==5\
+            } {
+                array set data [list $index,1 $readIOs $index,2 $writeIOs $index,3 $readBlocks $index,4 $writeBlocks]
+                set found($index) {}
+            }
+            for {set row 0} {$row<=3} {incr row} {
+                if {![info exists found($row)]} {
+                    array set data [list $row,1 0 $row,2 0 $row,3 0 $row,4 0]
+                }
+            }
+        } else {                                                                                    ;#  probably 2.2 or below kernel
+            scan [gets $statistics] {disk_rio %u %u %u %u} data(0,1) data(1,1) data(2,1) data(3,1)
+            scan [gets $statistics] {disk_wio %u %u %u %u} data(0,2) data(1,2) data(2,2) data(3,2)
+            scan [gets $statistics] {disk_rblk %u %u %u %u} data(0,3) data(1,3) data(2,3) data(3,3)
+            scan [gets $statistics] {disk_wblk %u %u %u %u} data(0,4) data(1,4) data(2,4) data(3,4)
+        }
         if {[info exists last]} {
             for {set row 0} {$row<=3} {incr row} {
                 for {set column 1} {$column<=4} {incr column} {
