@@ -1,14 +1,14 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: drag.tcl,v 2.2 1999/08/28 10:55:13 jfontain Exp $}
+set rcsId {$Id: drag.tcl,v 2.6 2000/12/31 23:14:19 jfontain Exp $}
 
 class dragSite {                                          ;# make a Tk widget a drag site with multiple formats support for its data
 
     if {![info exists (grabber)]} {
         # use a specific invisible frame so that when dragging is active, the frame is grabbed and its specific cursor is used,
         # thus preventing any interferences from source grab widget. use a specific cursor for user feedback
-        set ::dragSite::(grabber) $widget::([new frame . -background {} -width 0 -height 0 -cursor circle],path)
+        set (grabber) $widget::([new frame . -background {} -width 0 -height 0 -cursor circle],path)
         place $(grabber) -x -32768 -y -32768                                                         ;# make sure frame is invisible
     }
 
@@ -79,10 +79,10 @@ class dragSite {                                          ;# make a Tk widget a
         bindings::set $($this,bindings) <Button1-Motion> {}                                                         ;# reset binding
         set command $switched::($this,-validcommand)
         if {([string length $command]>0)&&![uplevel #0 $command $xWidget $yWidget]} return
-        set dragSite::(x) $xWidget                                ;# make widget and root coordinates available to provider commands
-        set dragSite::(y) $yWidget ;### Tcl bug ###
-        set dragSite::(X) $xRoot
-        set dragSite::(Y) $yRoot
+        set (x) $xWidget                                          ;# make widget and root coordinates available to provider commands
+        set (y) $yWidget
+        set (X) $xRoot
+        set (Y) $yRoot
         bindings::set $($this,bindings) <Button1-Motion> "dragSite::start $this %X %Y"
     }
 
@@ -96,11 +96,11 @@ class dragSite {                                          ;# make a Tk widget a
         update idletasks
 
         # for the highlighting frame, use a toplevel object to make sure its path is unique
-        set ::dragSite::(highlightFrame) [new toplevel . -background {} -highlightthickness 1 -highlightbackground black]
+        set (highlightFrame) [new toplevel . -background {} -highlightthickness 1 -highlightbackground black]
         wm withdraw $widget::($(highlightFrame),path)
         wm overrideredirect $widget::($(highlightFrame),path) 1
-        set dragSite::(dropRegions) [dropSite::regions [array names ${this}provider]]      ;# note: drop code must not invoke update
-        set dragSite::(lastSite) 0
+        set (dropRegions) [dropSite::regions [array names ${this}provider]]                ;# note: drop code must not invoke update
+        set (lastSite) 0
         # setup bindings after initializations above
         bind $(grabber) <ButtonRelease-1> "dragSite::drop $this %X %Y"
         bind $(grabber) <Button1-Motion> "dragSite::track $this %X %Y"
@@ -133,7 +133,7 @@ class dragSite {                                          ;# make a Tk widget a
             $frame configure -width [expr {[winfo width $path]+2}] -height [expr {[winfo height $path]+2}]
             showTopLevel $frame +[expr {[winfo rootx $path]-1}]+[expr {[winfo rooty $path]-1}]            ;# show highlighting frame
         }
-        set dragSite::(lastSite) $site
+        set (lastSite) $site
     }
 
     proc drop {this xRoot yRoot} {
@@ -146,10 +146,10 @@ class dragSite {                                          ;# make a Tk widget a
         update idletasks
 
         delete $(highlightFrame)
-        unset ::dragSite::(lastSite)
+        unset (lastSite)
 
         set site [dropSite $xRoot $yRoot]
-        unset ::dragSite::(dropRegions)
+        unset (dropRegions)
         if {($site==0)||[string equal $switched::($site,-path) $switched::($this,-path)]} {
             return                                                               ;# no point in being able to drop data in drag site
         }
@@ -158,7 +158,7 @@ class dragSite {                                          ;# make a Tk widget a
             if {[catch {set command [set ${this}provider($format)]}]} continue                           ;# skip unavailable formats
             set data($format) [uplevel #0 $command [list $format]]                   ;# invoke at global level as Tk buttons command
         }
-        unset ::dragSite::(x) ::dragSite::(y) ::dragSite::(X) ::dragSite::(Y)               ;# no longer needed by provider commands
+        unset (x) (y) (X) (Y)                                                               ;# no longer needed by provider commands
         dropSite::dropped $site                                                                    ;# tell drop site to process data
         unset data                                                              ;# free memory after data has been used by drop site
     }
