@@ -1,7 +1,8 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: freetext.tcl,v 2.1 1999/08/28 08:30:00 jfontain Exp $}
+set rcsId {$Id: freetext.tcl,v 2.17 2001/02/03 17:20:11 jfontain Exp $}
+
 
 class freeText {
 
@@ -28,24 +29,23 @@ class freeText {
 
     proc iconData {} {
         return {
-            R0lGODdhKAAoAIQAAHh4eMjMyAAAANDU0Pj8+Hh8eDg4OEBIQLi8uFAIYHAoeHAweGgYeIgQoLgo0KAYuLAYyLAg0GggeNg4+KgYyNhA+LgY2OBI+OBQ+Ngo
-            +AAAAAAAAAAAAAAAAAAAAAAAACwAAAAAKAAoAAAF/iAgjmRpnugYAELrvnAsz+0qBHgeDDi/6z6fbjhkBQjIJBKgbDKbyecSaTtCCVJo1ql82gZXQiEsJo+j
-            VKOowG673/B4W2UMn693aN7LAuPNgGgEVUg0hodYaTciMGSOTQJzX4Uvj5YEAmWDdZiVl46ZZ5OdLp+gI5uLLJ6mVwIGBwcAhKQtZLaWAqijjWEtCLm7nL21
-            lAi4xYXCqr2/x5mlts64rwYGs8OsCMC/0wLPwATbmMt+xDDgnelKuiKpVs3f4L/q9OzljJ703dLf+5mYYF2jda4fv3EvEC6Dx0rcuG0PwzkcR84dwYat7llc
-            JSOjqxc2Eogk5jGJAAULcxYwsNGgJcmSmFAucGDjAYQIL0sKkKBAwQQbFCBQOERUhswKQClEsMA0pdOnUKNKdXoBqAOmTadq3ZoSKQusWLmKlYrBBtizaNOq
-            RetVwNq3cMFWmGsjg927ePPq3at3boa6fAMLDgx4sGHDK4goXsyYcQgAOw==
+            R0lGODdhJAAkAIQAAPj8+Hh4eHh8eAAAANjc2Dg4OJicmICEgIiQiJCYkKCkoKisqLi8uMDEwMjQyNDY0ODk4Ojs6AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
+            AAAAAAAAAAAAAAAAAAAAAAAAACwAAAAAJAAkAAAF/iAgjmJAnuY5pqVKsiqMvu4o1MBd6y0QBIKgcEgsGoU/mQ3HUzWVpOZJGqWpBtisdsvd9n7CAWFMLpvP
+            5MFQJka73QMRFdB+29NyV/1uH5hSYEF7fHAFQyeDZViEfj56b1gGfI1QdVpjkYuYWWl/JUNtkQaLnKQDBqOhhkFsZKgEmaeps2OvlI+bWbSxkoqeV5uovLCn
+            kWa3PqDBxKliw5oDqwKtucS8klq2v4hlqJLe370E4JjbJImEx79c6X1bROjtmEQH9fXx8mr2Bwj9/fjt1PhDkKBgwS4IE2IRYDCBgocQj0iUCFHBgosYM2rc
+            yLEjg48gQ4ocSbJkg5MoK1OqXMmypYOXMGPKnEmz5oObOHPq3MmzJ4SfQIMKHUq0aISjSJMqXcq0aQgAOw==
         }
     }
 
     proc options {this} {
         # force size values
         return [list\
-            [list -cellindices cellIndices CellIndices {} {}]\
+            [list -cellindices {} {}]\
             [list -deletecommand {} {}]\
-            [list -draggable draggable Draggable 0 0]\
-            [list -endtext endText EndText {} {}]\
-            [list -height height Height 1]\
-            [list -width width Width 40]\
+            [list -draggable 0 0]\
+            [list -endtext {} {}]\
+            [list -height 1]\
+            [list -width 40]\
         ]
     }
 
@@ -69,7 +69,6 @@ class freeText {
         if {!$value} return                                                                                           ;# no dragging
         set ($this,drag) [new dragSite -path $widget::($this,path) -validcommand "freeText::validateDrag $this 0"]
         dragSite::provide $($this,drag) OBJECTS "freeText::dragData $this"
-
         set ($this,selector) [new selector -selectcommand "freeText::setLabelsState $this"]
     }
 
@@ -106,13 +105,11 @@ class freeText {
     }
 
     proc supportedTypes {this} {
-        return {ascii dictionary integer real}
+        return {ascii dictionary integer real clock}
     }
 
-    proc monitorCell {this array row column} {
+    proc monitorCell {this array row column} {                                                    ;# allow duplicate monitored cells
         viewer::registerTrace $this $array
-        set cell ${array}($row,$column)
-        if {[lsearch -exact [cellsFromLabels $this $($this,labels)] $cell]>=0} return                    ;# already displayed, abort
         set path $widget::($this,path)
         if {[info exists ($this,nextCellIndex)]} {                ;# recreate data cell labels placement from recorded configuration
             set index [lindex $composite::($this,-cellindices) $($this,nextCellIndex)]
@@ -126,10 +123,9 @@ class freeText {
             set index insert                                                ;# insert cell label text and window at insertion cursor
             $path insert $index "[viewer::label $array $row $column]: "
         }
-        set label [new label $path $cell]
+        set label [new label $path]
         set labelPath $label::($label,path)
-        # keep track of label existence
-        switched::configure $label -deletecommand "freeText::deletedLabel $this $array $label"
+        switched::configure $label -deletecommand "freeText::deletedLabel $this $array $label"      ;# keep track of label existence
         if {$composite::($this,-draggable)} {                                              ;# setup dragging and selection for label
             set drag [new dragSite -path $labelPath -validcommand "freeText::validateDrag $this $label"]
             dragSite::provide $drag OBJECTS "freeText::dragData $this"
@@ -143,15 +139,17 @@ class freeText {
         }
         lappend ($this,labels) $label
         $path window create $index -window $labelPath
-        set ($this,cell,$label) $cell
+        set ($this,cell,$label) ${array}($row,$column)
     }
 
     proc update {this array args} {                              ;# update display using cells data. ignore eventual trace arguments
         foreach label $($this,labels) {
-            if {[catch {set $($this,cell,$label)} value]} {                                                  ;# handle invalid cells
-                switched::configure $label -text ?
+            set cell $($this,cell,$label)
+            if {[string first $array $cell]<0} continue                                  ;# check that cell belongs to updated array
+            if {[info exists $cell]} {
+                switched::configure $label -text [set $cell]                                               ;# may be the ? character
             } else {
-                switched::configure $label -text $value
+                switched::configure $label -text ?
             }
         }
     }
@@ -171,7 +169,7 @@ class freeText {
         foreach label $labels {
             lappend cells $($this,cell,$label)
         }
-        return $cells
+        return $cells                                                                                      ;# may contain duplicates
     }
 
     proc cells {this} {
@@ -222,38 +220,46 @@ class freeText {
             append text $string
         }
         lappend options -endtext $text
-        set indices {}
         foreach {key path index} [$widget::($this,path) dump -window 1.0 end] {
             if {[string length $path]==0} continue                                                         ;# ignore deleted windows
-            lappend indices $index
+            set position($path) $index
         }
-        if {[llength $indices]>0} {
-            lappend options -cellindices $indices
+        if {[info exists position]} {
+            foreach label $($this,labels) {                                                          ;# get labels in creation order
+                lappend indices $position($label::($label,path))
+            }
+            lappend options -cellindices $indices                  ;# so that labels may be placed properly when reloading from file
         }
         return $options
     }
 
+    proc setCellColor {this array row column color} {
+        set cell ${array}($row,$column)
+        foreach label $($this,labels) {
+            if {[string equal $($this,cell,$label) $cell]} {
+                switched::configure $label -background $color
+            }                                                               ;# not done since there can be duplicate monitored cells
+        }
+    }
+
 }
 
 class freeText {
 
     class label {
 
-        proc label {this parentPath cell args} switched {$args} {
-            set label [new label $parentPath\
-                -font $font::(mediumBold) -relief sunken -padx 0 -pady 0 -borderwidth 1 -cursor top_left_arrow\
-            ]
+        proc label {this parentPath args} switched {$args} {
+            set label [new label $parentPath -font $font::(mediumBold) -padx 0 -pady 0 -borderwidth 1 -cursor left_ptr]
             # keep track of label existence as it may be deleted by directly editing in the parent text widget
             bind $widget::($label,path) <Destroy> "delete $this"
             set ($this,path) $widget::($label,path)
             set ($this,label) $label
-            set ($this,cell) $cell
             switched::complete $this
         }
 
         proc ~label {this} {
-            bind [set ($this,path)] <Destroy> {}                                                ;# remove binding to avoid recursion
-            delete [set ($this,label)]
+            bind $($this,path) <Destroy> {}                                                     ;# remove binding to avoid recursion
+            delete $($this,label)
             if {[string length $switched::($this,-deletecommand)]>0} {
                 uplevel #0 $switched::($this,-deletecommand)                                ;# always invoke command at global level
             }
@@ -261,22 +267,31 @@ class freeText {
 
         proc options {this} {
             return [list\
+                [list -background {}]\
                 [list -deletecommand {} {}]\
                 [list -text {} {}]\
             ]
         }
 
+        proc set-background {this value} {
+            if {[string length $value]==0} {
+                $($this,path) configure -background $widget::option(label,background)
+            } else {
+                $($this,path) configure -background $value
+            }
+        }
+
         proc set-deletecommand {this value} {}                                                   ;# data is stored at switched level
 
         proc set-text {this value} {
-            [set ($this,path)] configure -text $value
+            $($this,path) configure -text $value
         }
 
         proc select {this select} {
             if {$select} {
-                [set ($this,path)] configure -background white
+                $($this,path) configure -relief sunken
             } else {
-                [set ($this,path)] configure -background $widget::(default,ButtonBackgroundColor)
+                $($this,path) configure -relief flat
             }
         }
 
