@@ -1,7 +1,7 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: getopt.tcl,v 1.5 1998/12/26 10:43:44 jfontain Exp $}
+set rcsId {$Id: getopt.tcl,v 2.3 2000/12/31 23:14:19 jfontain Exp $}
 
 # switches is a list of switch / flag value where switch is the actual expected string (which must start by either - or +) and flag
 # is a boolean that tell whether an argument is required.
@@ -35,7 +35,7 @@ proc parseCommandLineArguments {switches arguments arrayName} {
     for {set index 0} {$index<$maximum} {incr index} {
         set switch $argument($index)
         if {![info exists flag($switch)]} break                                                                   ;# end of switches
-        if {[string compare $switch --]==0} {                                                                     ;# end of switches
+        if {[string equal $switch --]} {                                                                          ;# end of switches
             incr index                                                                                ;# skip end of switches marker
             break
         }
