@@ -1,17 +1,17 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: help.tcl,v 1.32 1999/08/28 08:31:42 jfontain Exp $}
+set rcsId {$Id: help.tcl,v 2.10 2000/12/31 23:14:19 jfontain Exp $}
 
 namespace eval help {
 
-    variable nameAndVersion "
-moodss: a Modular Object Oriented Dynamic SpreadSheet
-version $global::applicationVersion
-    "
+    variable nameAndVersion \
+"moodss
+Modular Object Oriented Dynamic SpreadSheet
+version $global::applicationVersion"
 
     variable description {
-Copyright (C) 1997-1999 Jean-Luc Fontaine. All rights reserved.
+Copyright (C) 1997-2001 Jean-Luc Fontaine. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 
@@ -19,16 +19,16 @@ This program is distributed in the hope that it will be useful, but WITHOUT ANY
 
 (see Copyright under the Help menu for more details)
 
-        jfontain@multimania.com
+        jfontain@free.fr
 
 
 This software uses the following extensions:
 
     - the BLT library by George Howlett
     - the tkTable widget by Jeffrey Hobbs
-    - the wcb (Widget Callback Package) library by Csaba Nemethi
+    - the MIME/SMTP library by Marshall Rose
 
-Thank you so very much to the authors for their great work.
+Thank you so very much to these people for their great work.
 
 
 This software also includes the Tcl HTML library developped by Sun Microsystems and made available under the following license terms:
@@ -127,13 +127,16 @@ proc aboutDialogBox {} {
     $composite::($text,scrolled,path) configure\
         -state disabled -borderwidth 0 -font $font::(mediumNormal) -wrap word -height 20 -padx 10
 
-    grid rowconfigure $frame 1 -weight 1
     grid columnconfigure $frame 0 -weight 1
     grid columnconfigure $frame 1 -weight 1
-    grid $widget::($text,path) -row 1 -column 0 -columnspan 2 -sticky nsew
+    grid rowconfigure $frame 0 -weight 0
+    grid rowconfigure $frame 1 -weight 0
+    grid rowconfigure $frame 2 -weight 1
 
-    grid [label $frame.icon -image [image create photo -data [dataGraph::iconData]]] -row 0 -column 0
+    grid [label $frame.icon -image [image create photo -data [dataGraph::iconData]] -relief sunken] -row 0 -column 0 -padx 2
     grid [label $frame.name -text $help::nameAndVersion] -row 0 -column 1
+    grid [frame $frame.separator -relief sunken -borderwidth 1 -height 2] -row 1 -column 0 -columnspan 2 -sticky ew
+    grid $widget::($text,path) -row 2 -column 0 -columnspan 2 -sticky nsew
 
     bind $widget::($dialog,path) <Destroy> "catch {delete $text}"                    ;# delete objects not managed by the dialog box
     dialogBox::display $dialog $frame
@@ -161,3 +164,10 @@ proc versionsDialogBox {} {
     incr ::versionsData(updates)
     dialogBox::display $dialog $widget::($table,path)
 }
+
+proc linkedHelpWidgetTip {path} {
+    return [new widgetTip\
+        -path $path\
+        -text "display the relevant section in the\ngeneral help window (may take some\ntime if the window is not yet opened)"\
+    ]
+}
