@@ -33,6 +33,11 @@
 
 # These are Defined in HTML 2.0
 
+
+
+set ::htmlLibraryCode {
+
+
 array set HMtag_map {
 	b      {weight bold}
 	blockquote	{style i indent 1 Trindent rindent}
@@ -426,7 +431,7 @@ proc HMtag_a {win param text} {
 proc HMgoto {win where {callback HMwent_to}} {
 	upvar #0 HM$win var
 	if {[regexp N:$where [$win mark names]]} {
-		$win see N:$where
+		$win yview N:$where
 		update
 		eval $callback $win [list $where]
 		return 1
@@ -1390,29 +1395,4 @@ proc HMcgiMap {data} {
 	}
 }
 
-# There is a bug in the tcl library focus routines that prevents focus
-# from every reaching an un-viewable window.  Use our *own*
-# version of the library routine, until the bug is fixed, make sure we
-# over-ride the library version, and not the otherway around
-
-auto_load tkFocusOK
-proc tkFocusOK w {
-    set code [catch {$w cget -takefocus} value]
-    if {($code == 0) && ([string length $value]>0)} {
-    if {$value == 0} {
-        return 0
-    } elseif {$value == 1} {
-        return 1
-    } else {
-        set value [uplevel #0 $value $w]
-        if {[string length $value]>0} {
-        return $value
-        }
-    }
-    }
-    set code [catch {$w cget -state} value]
-    if {($code == 0) && [string equal $value disabled]} {
-    return 0
-    }
-    regexp Key|Focus "[bind $w] [bind [winfo class $w]]"
 }
