@@ -1,7 +1,7 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: keyslink.tcl,v 1.7 1998/12/26 10:43:44 jfontain Exp $}
+set rcsId {$Id: keyslink.tcl,v 2.3 2000/12/31 23:14:19 jfontain Exp $}
 
 # simple class to safely add key bindings to a button so that if follows the keys move:
 # sinks when keys is pressed, raises when key is released
@@ -21,9 +21,9 @@ class buttonKeysLink {
             bindings::set $bindings <KeyRelease-$key> "$buttonPath configure -relief raised"
         }
         bindings::set $bindings <Destroy> "delete $this"                                              ;# self destruct before target
-        set buttonKeysLink::($this,bindings) $bindings
+        set ($this,bindings) $bindings
     }
     proc ~buttonKeysLink {this} {
-        delete $buttonKeysLink::($this,bindings)
+        delete $($this,bindings)
     }
 }
