@@ -1,21 +1,25 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: lifolbl.tcl,v 2.1 1999/08/28 08:33:26 jfontain Exp $}
+set rcsId {$Id: lifolbl.tcl,v 2.9 2000/12/31 23:14:19 jfontain Exp $}
+
 
 # last-in-first-out label generally used at the bottom of an application window for messages display
 
-source xifo.tcl
-
-class lifoLabel {}
+class lifoLabel {
+    variable header [mc Message:]
+}
 
 proc lifoLabel::lifoLabel {this parentPath args} composite {
-    [new frame $parentPath -relief sunken -borderwidth $widget::(default,ButtonBorderWidth)] $args
+    [new frame $parentPath -relief sunken -borderwidth $widget::option(button,borderwidth)] $args
 } {
+    variable header
+
     set path $widget::($this,path)
-    composite::manage $this [new label $path -font {helvetica -12 bold} -text Message:] header [new frame $path] separator\
-        [new label $path -font {helvetica -12} -anchor w] body
-    pack $composite::($this,header,path) $composite::($this,separator,path) -side left
+    composite::manage $this [new label $path -font {helvetica -12 bold} -text $header] header [new frame $path] separator\
+        [new scrollingLabel $path -font {helvetica -12}] body
+    # anchor header for proper behavior with multi-line messages
+    pack $composite::($this,header,path) $composite::($this,separator,path) -side left -anchor nw
     pack $composite::($this,body,path) -fill x -expand 1
     set ($this,lifo) [new lifo]
     composite::complete $this
@@ -26,13 +30,15 @@ proc lifoLabel::~lifoLabel {this} {
 }
 
 proc lifoLabel::options {this} {
+    variable header
+
     return [list\
-        [list -borderwidth borderWidth BorderWidth $widget::(default,ButtonBorderWidth) $widget::(default,ButtonBorderWidth)]\
-        [list -font font Font {helvetica -12} {helvetica -12}]\
-        [list -header header Text Message: Message:]\
-        [list -headerfont headerFont Font {helvetica -12 bold} {helvetica -12 bold}]\
-        [list -relief relief Relief sunken sunken]\
-        [list -spacing spacing Spacing 0 0]\
+        [list -borderwidth $widget::option(button,borderwidth) $widget::option(button,borderwidth)]\
+        [list -font {helvetica -12} {helvetica -12}]\
+        [list -header $header $header]\
+        [list -headerfont {helvetica -12 bold} {helvetica -12 bold}]\
+        [list -relief sunken sunken]\
+        [list -spacing 0 0]\
     ]
 }
 
@@ -41,7 +47,7 @@ foreach option {-borderwidth -relief} {
 }
 
 proc lifoLabel::set-font {this value} {
-    $composite::($this,body,path) configure -font $value
+    composite::configure $composite::($this,body) -font $value
 }
 
 proc lifoLabel::set-headerfont {this value} {
@@ -57,16 +63,16 @@ proc lifoLabel::set-spacing {this value} {
 }
 
 proc lifoLabel::push {this string} {                                                          ;# user procedure for pushing a string
-    if {[string length [set current [$composite::($this,body,path) cget -text]]]>0} {  ;# store current string in stack if non empty
-        xifo::in $($this,lifo) $current
+    if {[string length [set current [composite::cget $composite::($this,body) -text]]]>0} {
+        xifo::in $($this,lifo) $current                                                ;# store current string in stack if non empty
     }
-    $composite::($this,body,path) configure -text $string
+    composite::configure $composite::($this,body) -text $string
 }
 
 proc lifoLabel::pop {this} {                                                                  ;# user procedure for popping a string
     set string {}
     catch {set string [lifo::out $($this,lifo)]}
-    $composite::($this,body,path) configure -text $string                                           ;# set label text (may be empty)
+    composite::configure $composite::($this,body) -text $string
     return $string
 }
 
