@@ -1,11 +1,11 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: minimal.tcl,v 1.1 1999/01/03 15:34:15 jfontain Exp $}
+set rcsId {$Id: minimal.tcl,v 1.6 2001/01/14 13:11:31 jfontain Exp $}
 
-# the absolute minimal module file (see docuemented code in random.tcl module file or HTML documentation)
+# the absolute minimal module file (see documented code in random.tcl module file or HTML documentation)
 
-package provide minimal 1.0
+package provide minimal [lindex {$Revision: 1.6 $} 1]
 
 namespace eval minimal {
 
