@@ -1,12 +1,12 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: mounts.tcl,v 1.10 1999/08/27 20:03:13 jfontain Exp $}
+set rcsId {$Id: mounts.tcl,v 2.10 2001/01/27 15:08:19 jfontain Exp $}
 
 
-package provide mounts 1.5
-package require filesystem
-package require network 1.3
+package provide mounts [lindex {$Revision: 2.10 $} 1]
+package require filesystem 1
+package require network 1
 
 namespace eval mounts {
 
@@ -22,7 +22,7 @@ namespace eval mounts {
         sort {0 increasing}
         switches {-r 1 --remote 1}
     }
-    set file [open [file join $::packageDirectory(mounts) mounts.htm]]
+    set file [open mounts.htm]
     set data(helpText) [read $file]                                                           ;# initialize HTML help data from file
     close $file
 
@@ -33,18 +33,25 @@ namespace eval mounts {
 
         if {![catch {set locator $options(--remote)}]||![catch {set locator $options(-r)}]} {                   ;# remote monitoring
             set data(pollTimes) {30 10 20 60 120 300 600}                                ;# poll less often when remotely monitoring
-            foreach {remote(user) remote(host)} [network::parseRemoteLocator $locator] {}
-            network::checkRemoteOutputEmptiness $remote(host)
+            foreach {remote(protocol) remote(user) remote(host)} [network::parseRemoteLocator $locator] {}
+            network::checkRemoteOutputEmptiness $remote(protocol) $remote(user) $remote(host)
             set data(identifier) mounts($remote(host))
-            set file [open "| /usr/bin/rsh -nl $remote(user) $remote(host) cat /proc/version"]
+            set file [open "| /usr/bin/$remote(protocol) -n -l $remote(user) $remote(host) cat /proc/version"]
             fileevent $file readable {set ::mounts::remote(busy) 0}
             vwait ::mounts::remote(busy)                                                               ;# do not hang user interface
         } else {
             set data(pollTimes) {20 5 10 30 60 120 300 600}
             set file [open /proc/version]
         }
-        regexp {^[0-9\.]+} [lindex [gets $file] 2] version                         ;# ignore extra characters, such as in 2.2.0-pre1
-        close $file
+        regexp {^[\d\.]+} [lindex [gets $file] 2] version                          ;# ignore extra characters, such as in 2.2.0-pre1
+        if {[info exists remote]} {
+            # detect errors early (but avoid write on pipe with no readers errors by reading whole data)
+            if {[catch {read $file} message]||[catch {close $file} message]} {
+                error "on remote host $remote(host) as user $remote(user): $message"
+            }
+        } else {
+            close $file
+        }
         if {[package vcompare $version 2]<0} {                                                               ;# check kernel version
             puts stderr {mounts module needs at least a version 2 kernel}
             exit 1
@@ -56,8 +63,10 @@ namespace eval mounts {
     proc update {} {
         variable remote
 
-        if {[info exists remote]&&!$remote(busy)} {
-            remoteUpdate
+        if {[info exists remote]} {
+            if {!$remote(busy)} {
+                remoteUpdate
+            }
         } else {
             localUpdate
         }
@@ -69,8 +78,8 @@ namespace eval mounts {
         set file [open /etc/mtab]                        ;# must be opened each poll otherwise unmounted filesystems never disappear
         while {[gets $file line]>=0} {
             foreach {device directory type flags dump order} $line {}
-            if {[string compare $device none]==0} continue                                 ;# ignore irrelevant entries such as proc
             foreach {size total available used} [filesystem::statistics $directory] {}
+            if {$available==0} continue                              ;# ignore special cases, such as proc, pts, automount, usb, ...
             set ratio [expr {$size/1024.0}]                    ;# take filesystem block size into account to get values in kilobytes
             updateEntryData $directory $device $type [expr {round($total*$ratio)}] [expr {round($used*$ratio)}]\
                 [expr {round($available*$ratio)}] [expr {round(($used*100.0)/($used+$available))}]                      ;# used in %
@@ -84,7 +93,9 @@ namespace eval mounts {
     proc remoteUpdate {} {
         variable remote
 
-        set file [open "| /usr/bin/rsh -nl $remote(user) $remote(host) /bin/df -T"]
+        set remote(busy) 1
+        # use POSIX formatting to avoid line breaks
+        set file [open "| /usr/bin/$remote(protocol) -n -l $remote(user) $remote(host) /bin/df -PT"]
         fileevent $file readable "mounts::remoteUpdated $file"               ;# do not hang user interface and other modules updates
     }
 
@@ -94,11 +105,18 @@ namespace eval mounts {
 
         gets $file                                                                                               ;# skip header line
         while {[gets $file line]>=0} {
+            lappend lines $line
+        }
+        read $file                                           ;# avoid write on pipe with no readers errors by reading remaining data
+        if {[catch {close $file} message]} {
+            flashMessage "mounts error: $message"
+            set lines {}                                                                       ;# consider data corrupted as a whole
+        }
+        foreach line $lines {
             foreach {device type size used available capacity directory} $line {}
             updateEntryData $directory $device $type $size $used $available [string trimright $capacity %]
             set current($directory) {}
         }
-        close $file                                                                ;### errors and timeouts could be caught here ###
         cleanupEntriesData current
         set remote(busy) 0
         incr data(updates)
