@@ -1,24 +1,22 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: preferen.tcl,v 1.7 1999/08/08 18:31:40 jfontain Exp $}
+set rcsId {$Id: preferen.tcl,v 1.13 2000/12/31 23:14:19 jfontain Exp $}
+
 
 namespace eval preferences {
-
     # there is no knowledge in this file about which global variables are handled in preferences: only each folder namespace code
     # in the configuration interfeace knows and deals with them
 
     variable rcFileName ~/.moodssrc
-    variable variables {}
 
     proc read {} {
         variable rcFileName
-        variable variables
 
         if {![file readable $rcFileName]} {
             return {}
         }
-        set file [::open $rcFileName] ;### detect errors and report failures to user ###
+        set file [::open $rcFileName]
         set list {}
         foreach {name value} [::read $file] {
             set name [namespace tail $name]     ;# moodss versions before 8.1 wrote fully qualified names (such as global::variable)
@@ -33,12 +31,15 @@ namespace eval preferences {
         # at this point, variables in this namespace have been updated in the configuration interface
         variable rcFileName
 
-        set file [::open $rcFileName w] ;### detect errors and report failures to user ###
-        set list {}
-        foreach name $variables {
-            lappend list $name [set ::preferences::$name]
+        if {[catch {::open $rcFileName w} file]} {
+            tk_messageBox -type ok -default ok -icon error -message $file
+            return
         }
-        puts -nonewline $file $list
+        set data [record::globalData]
+        foreach name $variables {
+            append data [list $name [set ::preferences::$name]]\n
+        }
+        puts -nonewline $file $data
         close $file
     }
 
