@@ -1,14 +1,14 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: random.tcl,v 1.26 1999/07/20 20:25:29 jfontain Exp $}
+set rcsId {$Id: random.tcl,v 1.40 2001/01/27 17:42:00 jfontain Exp $}
 
 # module sample file with random data
 
 # corresponding pkgIndex.tcl file content:
 # package ifneeded random 4.0 "source [file join $dir random.tcl]"
 
-package provide random 4.3
+package provide random [lindex {$Revision: 1.40 $} 1]
 
 namespace eval random {                                                                    ;# namespace name must match package name
     # Module data is arranged in a single array that can be used by many views, such as tables, graphs, pies,...
@@ -21,27 +21,31 @@ namespace eval random {
     # Column numbers must be positive, starting from 0 included. Column members are:
     #   - label: the title displayed on the top cell of the data column.
     #   - type: valid types are those that the lsort command can handle.
-    #   - message: a short help string that appears in the message area at the bottom of the data window when the mouse cursor
-    #     passes over the column title cell.
+    #   - message: help text that appears in a widget tip when the mouse cursor is left over the column title cell for a short
+    #     while.
     #   - anchor: optional member. Column data is either centered by default (center), tucked to the left (left) or right side
     #     (right) of the column.
     # - pollTimes: in seconds. A list with the default poll time in first position. The other times can be in any order, as the core
     #   will sort them and use the lowest value as the minimum poll time value.
     #   The minimum poll seconds must be a reasonable value depending on the processing time of the update procedure.
-    # - visibleColumns: an optional list that specifies the table columns that should be displayed. If not specified, all the table
-    #   columns are visible.
-    # - columns: total number of data columns to be displayed (obsolete, no longer required).
-    # - sort: a list that defines the initial column to be used for sorting the table data and in which direction. The specified
-    #   column must be visible (see visibleColumns member above).
+    # - indices: an optional list that specifies the data columns that should be displayed. If not specified, all the data columns
+    #   are visible. Note: visibleColumns, the previous keyword for that function, is OBSOLETE (but still supported) after moodss
+    #   versions 8.22 and 11.7.
+    # - columns: total number of data columns to be displayed (OBSOLETE, no longer required).
+    # - sort: an optional list that defines the initial column to be used for sorting the table data and in which direction. The
+    #   specified column must be visible (see indices member above).
     # - indexColumns: a list that specifies the columns required to uniquely identify a row in the table. It is optional and
     #   defaults to the column 0, unless of course you do not have a 0 column index, in which case it is mandatory to specify this
-    #   list. When specified, all the columns in the list must be visible (see visibleColumns member above).
+    #   list. When specified, all the columns in the list must be visible (see indices member above and in views below).
     # - helpText: displayed when the module help is launched from the main window help menu. Can be plain text or HTML formatted
     #   (<HTML> and <BODY> tags required), in which case it is properly rendered in the module help window (note: tables, frames,
     #   and may other tags are not supported: stick to formatted text at the moment).
     # - views: optional. If specified, it defines one or more views to be used in place of the default view. One table will be
     #   displayed per view. Each view is a list of array members, suitable as an argument to an "array set" command. For each view,
-    #   2 members must be defined: visibleColumns and sort (syntax and usage are identical to the default table members).
+    #   1 member must be defined: indices (syntax and usage identical to the corresponding table member).
+    #   2 members are optional: sort (syntax and usage identical to the corresponding table member), and swap, a boolean, specifies
+    #   whether the data is displayed in a table with columns and rows swapped (useful when data has only a single or a couple of
+    #   rows permanently).
     # - switches: optional. A list of switch / boolean pairs. A switch is a single letter or a string prepended with the - or +
     #   sign. The boolean value (0 or 1) specifies whether the switch takes an argument. If the switches member exists, an
     #   appropriate initialize procedure must be provided by the module (see initialize procedure example in this module).
@@ -58,15 +62,14 @@ namespace eval random {
         3,label memory 3,type integer 3,message {memory usage in kilobytes}
         4,label command 4,type dictionary 4,message {command name} 4,anchor left
         pollTimes {10 5 20 30 60 120 300}
-        sort {1 decreasing}
         indexColumns {0 4}
         views {
-            {visibleColumns {0 1 3 4} sort {1 decreasing}}
-            {visibleColumns {0 2 4} sort {2 decreasing}}
+            {indices {0 1 3 4} sort {1 decreasing}}
+            {indices {0 2 4} sort {2 decreasing}}
         }
         switches {-a 0 --asynchronous 0 -i 0 --identify 0}
     }
-    set file [open [file join $::packageDirectory(random) random.htm]]                      ;# directory is set in pkgIndex.tcl file
+    set file [open random.htm]                                               ;# module directory is current during the loading phase
     set data(helpText) [read $file]                                   ;# initialize HTML help data from file in module sub-directory
     close $file
 
@@ -104,11 +107,18 @@ namespace eval random {
         }
     }
 
+    # The terminate procedure if it exists is invoked by the core when the module is unloaded dynamically.
+    proc terminate {} {
+        # We could do some cleanup chores right here.
+    }
+
     # The dynamic data array index is the row number followed by the column number separated by a comma.
     # The column number must start from 0 up to the total number of columns minus 1 (no holes are allowed in the column sequence)
     # The row number can take any positive integer value (between 0 and 2147483647) and be defined in any order, as long as it is
     # unique during the lifetime of the module data. if a new row is created, it must take a value that was never used: thus, the
     # number of a row that has disappeared is not allowed. Row numbers need not be consecutive.
+    # Voidness for numeric data cells (integer or real type) takes the form of the ? character. Reminder: as long as a data row
+    # exists, all its data cells must exist. Thus voidness cannot be expressed by non-existence.
 
     # The update procedure is mandatory only for synchronous modules, as it is never invoked by the core for asynchronous modules.
     # Its function is to update module data.
@@ -117,13 +127,13 @@ namespace eval random {
         variable asynchronous
 
         array set data "
-            0,0 john 0,1 [format %.1f [expr {rand()*30}]] 0,2 [expr {100+int(rand()*50)}] 0,3 [expr {10+int(rand()*50)}] 0,4 cc
-            1,0 bill 1,1 [format %.1f [expr {rand()*3}]] 1,2 [expr {300+int(rand()*100)}] 1,3 [expr {30+int(rand()*100)}] 1,4 xedit
-            2,0 anny 2,1 [format %.1f [expr {rand()*5}]] 2,2 [expr {200+int(rand()*30)}] 2,3 [expr {20+int(rand()*30)}] 2,4 ps
-            4,0 peter 4,1 [format %.1f [expr {rand()*8}]] 4,2 [expr {50+int(rand()*10)}] 4,3 [expr {5+int(rand()*10)}] 4,4 ls
-            6,0 laura 6,1 [format %.1f [expr {rand()*20}]] 6,2 [expr {90+int(rand()*20)}] 6,3 [expr {9+int(rand()*20)}] 6,4 emacs
-            3,0 robert 3,1 [format %.1f [expr {rand()*10}]] 3,2 [expr {500+int(rand()*150)}] 3,3 [expr {50+int(rand()*150)}] 3,4 top
-            9,0 laura 9,1 [format %.1f [expr {rand()*30}]] 9,2 [expr {100+int(rand()*50)}] 9,3 [expr {10+int(rand()*50)}] 9,4 cc
+            0,0 John\\nWo 0,1 [format %.1f [expr {rand()*30}]] 0,2 [expr {100+int(rand()*50)}] 0,3 [expr {10+int(rand()*50)}] 0,4 cc
+            1,0 Bill 1,1 [format %.1f [expr {rand()*3}]] 1,2 [expr {300+int(rand()*100)}] 1,3 [expr {30+int(rand()*100)}] 1,4 xedit
+            2,0 Anny\\nDoe 2,1 [format %.1f [expr {rand()*5}]] 2,2 [expr {200+int(rand()*30)}] 2,3 [expr {20+int(rand()*30)}] 2,4 ps
+            4,0 Peter\\nWard 4,1 [format %.1f [expr {rand()*8}]] 4,2 [expr {50+int(rand()*10)}] 4,3 [expr {5+int(rand()*10)}] 4,4 ls
+            6,0 Laura 6,1 [format %.1f [expr {rand()*20}]] 6,2 [expr {90+int(rand()*20)}] 6,3 [expr {9+int(rand()*20)}] 6,4 emacs
+            3,0 Robert 3,1 [format %.1f [expr {rand()*10}]] 3,2 [expr {500+int(rand()*150)}] 3,3 [expr {50+int(rand()*150)}] 3,4 top
+            9,0 Laura 9,1 [format %.1f [expr {rand()*30}]] 9,2 [expr {100+int(rand()*50)}] 9,3 [expr {10+int(rand()*50)}] 9,4 cc
         "
         incr data(updates)                         ;# write variable so that update can be detected with an eventual tcl write trace
         if {$asynchronous} {                                                           ;# eventually simulate asynchronous operation
