@@ -1,11 +1,11 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: route.tcl,v 1.16 1999/08/27 20:02:23 jfontain Exp $}
+set rcsId {$Id: route.tcl,v 2.9 2001/01/27 15:08:19 jfontain Exp $}
 
 
-package provide route 1.4
-package require network 1.3
+package provide route [lindex {$Revision: 2.9 $} 1]
+package require network 1
 
 namespace eval route {
 
@@ -22,7 +22,7 @@ namespace eval route {
         sort {1 increasing}
         switches {-n 0 --numeric 0 -r 1 --remote 1}
     }
-    set file [open [file join $::packageDirectory(route) route.htm]]
+    set file [open route.htm]
     set data(helpText) [read $file]                                                           ;# initialize HTML help data from file
     close $file
 
@@ -36,15 +36,15 @@ namespace eval route {
         set lookup [expr {![info exists options(-n)]&&![info exists options(--numeric)]}]            ;# host or network names lookup
         if {![catch {set locator $options(--remote)}]||![catch {set locator $options(-r)}]} {                   ;# remote monitoring
             set data(pollTimes) {20 10 30 60 120 300 600}                                ;# poll less often when remotely monitoring
-            foreach {remote(user) remote(host)} [network::parseRemoteLocator $locator] {}
-            network::checkRemoteOutputEmptiness $remote(host)
+            foreach {remote(protocol) remote(user) remote(host)} [network::parseRemoteLocator $locator] {}
+            network::checkRemoteOutputEmptiness $remote(protocol) $remote(user) $remote(host)
             set data(identifier) route($remote(host))
-            set file [open "| /usr/bin/rsh -nl $remote(user) $remote(host) cat /proc/net/route"]
+            set file [open "| /usr/bin/$remote(protocol) -n -l $remote(user) $remote(host) cat /proc/net/route"]
             fileevent $file readable {set ::route::remote(busy) 0}
             vwait ::route::remote(busy)                                                                     ;# allow GUI interaction
-            if {[catch {close $file} message]} {
-                puts stderr "on remote host $remote(host) as user $remote(user): $message"
-                exit 1                                                                                        ;# detect errors early
+            # detect errors early (but avoid write on pipe with no readers errors by reading whole data)
+            if {[catch {read $file} message]||[catch {close $file} message]} {
+                error "on remote host $remote(host) as user $remote(user): $message"
             }
         } else {
             set data(pollTimes) {10 5 20 30 60 120 300 600}
@@ -66,7 +66,7 @@ namespace eval route {
             if {![info exists routeFile]} {                                ;# start data gathering process in a non blocking fashion
                 if {$remote(busy)} return                                           ;# core invocation while waiting for remote data
                 set remote(busy) 1
-                set file [open "| /usr/bin/rsh -nl $remote(user) $remote(host) cat /proc/net/route"]
+                set file [open "| /usr/bin/$remote(protocol) -n -l $remote(user) $remote(host) cat /proc/net/route"]
                 # do not hang GUI, allow other modules updates
                 fileevent $file readable "set ::route::routeFile $file; ::route::update"
                 return                                                                                       ;# wait for remote data
@@ -76,6 +76,18 @@ namespace eval route {
         }
         gets $routeFile                                                                                          ;# skip header line
         while {[gets $routeFile line]>=0} {
+            lappend lines $line
+        }
+        if {[info exists remote]} {                                                 ;# closing is necessary since seek does not work
+            read $routeFile                                  ;# avoid write on pipe with no readers errors by reading remaining data
+            if {[catch {close $routeFile} message]} {                                    ;# communication error can be detected here
+                flashMessage "route error: $message"
+                set lines {}                                                                   ;# consider data corrupted as a whole
+            }
+            unset routeFile
+            set remote(busy) 0
+        }
+        foreach line $lines {
             scan $line {%s %2x%2x%2x%2x %2x%2x%2x%2x %x %u %u %u %2x%2x%2x%2x}\
                 interface destination(3) destination(2) destination(1) destination(0)\
                 gateway(3) gateway(2) gateway(1) gateway(0) flags references use metric mask(3) mask(2) mask(1) mask(0)
@@ -85,7 +97,7 @@ namespace eval route {
                 incr nextIndex
             }
             set data($row,0) $interface
-            if {[string compare $address 0.0.0.0]==0} {                                                             ;# default route
+            if {[string equal $address 0.0.0.0]} {                                                                  ;# default route
                 set data($row,1) default
             } else {
                 set data($row,1) $address
@@ -94,7 +106,7 @@ namespace eval route {
                 }
             }
             set data($row,2) $gateway(0).$gateway(1).$gateway(2).$gateway(3)
-            if {[string compare $data($row,2) 0.0.0.0]==0} {
+            if {[string equal $data($row,2) 0.0.0.0]} {
                 set data($row,2) {}                                                                                    ;# no gateway
             } else {
                 if {$lookup} {
@@ -103,7 +115,7 @@ namespace eval route {
             }
             set data($row,3) [network::routeletterflags $flags]
             set data($row,4) $mask(0).$mask(1).$mask(2).$mask(3)
-            if {[string compare $data($row,4) 0.0.0.0]==0} {
+            if {[string equal $data($row,4) 0.0.0.0]} {
                 set data($row,4) {}                                                                                       ;# no mask
             }
             set data($row,5) $references
@@ -115,11 +127,6 @@ namespace eval route {
             if {[info exists current($address)]} continue
             unset index($address) data($row,0) data($row,1) data($row,2) data($row,3)
         }
-        if {[info exists remote]} {                                                 ;# closing is necessary since seek does not work
-            close $routeFile
-            unset routeFile
-            set remote(busy) 0
-        }
         incr data(updates)
     }
 }
