@@ -1,12 +1,12 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: sumtable.tcl,v 2.1 1999/08/28 09:22:49 jfontain Exp $}
+set rcsId {$Id: sumtable.tcl,v 2.19 2001/01/12 21:45:31 jfontain Exp $}
+
 
 class summaryTable {
 
-    set ::summaryTable::(nextDataIndex) 0            ;# used when data array index is not specified as an option when creating table
-    set ::summaryTable::(void) 000000        ;# must look void but be a valid number since it may be dropped in another viewer as is
+    set (nextDataIndex) 0                            ;# used when data array index is not specified as an option when creating table
 
     proc summaryTable {this parentPath args} composite {[new frame $parentPath] $args} viewer {} {
         composite::complete $this
@@ -16,7 +16,7 @@ class summaryTable {
         array set $($this,dataName) {
             updates 0
             0,label data 0,type ascii 0,message {data cell description}
-            1,label current 1,type real 1,message {current value (000000 means void)}
+            1,label current 1,type real 1,message {current value}
             2,label average 2,type real 2,message {average value since viewer creation}
             3,label minimum 3,type real 3,message {minimum value since viewer creation}
             4,label maximum 4,type real 4,message {maximum value since viewer creation}
@@ -37,6 +37,8 @@ class summaryTable {
         if {$composite::($this,-draggable)} {
             # extend data table drag capabilities ### hack ### also eventually allow row selection only instead of cells ###
             dragSite::provide $dataTable::($table,drag) OBJECTS "summaryTable::dragData $this"
+            # intercept original data cells dragging
+            dragSite::provide $dataTable::($table,drag) DATACELLS "summaryTable::dragData $this"
         }
         pack $widget::($table,path) -fill both -expand 1
         set ($this,dataTable) $table
@@ -60,23 +62,22 @@ class summaryTable {
 
     proc iconData {} {
         return {
-            R0lGODdhKAAoAIQAAHt5e87PzgAAANbX1v///9/f339/fzk4OUJJQlIIY2sQe2sYe4wQpXMoe70o1qUYvbUYzrUg1msge944/60YznMwe95B/+dJ/70Y3udR
-            /9YY984Y794o/wAAAAAAAAAAACwAAAAAKAAoAAAF/iAgjmRpnugYAELrvnAsz+0qBHgeDDi/6z6fbjhkBQjIJBKgbDKbyecSaTtCCVJo1ql82gZImniMpRpF
-            sELBoG672e53QUCqhl9qeDy/b7MFZQRfdy58fWuHiIBeRoQtBpCRkpOUkXQigmcsLwSInZ8FnWygoH8HCAcAgwRpBAakoaGvsaSvl0x2rJyetLGjvaJzI5kC
-            YLoulcnKt8Qrm4WIh3p7pggIqo3HLYZ903F/w6tp0d2J4Ji5MMrrk8xVaLu/sPK9pgep6XiusJ+z/LbhWBiDEYwfr3nC0GVTx66hO4HwoHmTI23OKXwL8ZCj
-            Zi4hrowSO1Z8eMORgIYOlgPSKAjsYL05L2wkmNnKHzCbtVgpWMDTBoOfBF2WahlMQIOjDmw8gBCBIcplEo5OsEEBAoVxE/10NFqhggWqFJpqzMqNI9cKF6g6
-            wIBVZDkBURt8ZYGh7pi7Mhp0zWCjrl8MXQMLHky4cGAMff8CNsy4cVfELDRs4ECZg+PLhTnYqMy5s+fPoDlvDk26tOcVRFKrXr06BAA7
+            R0lGODdhJAAkAOMAAPj8+Hh4eAAAAHh8eNjc2ICEgIiQiJCYkKCkoKisqLi8uMDEwMjQyNDY0ODk4Ojs6CwAAAAAJAAkAAAE/hDIKQO99s5cNeUaiH2aYJ5o
+            qqZbFQyDQBDDbN/1jRMmDIMymm43nNUEg84lmCs2h8ckQARA+q7YLBapDLxiAGF4TAjXyOSj9UeRAZLl+BiOLie505JZzj/z93hUa1qEWYEuMExFRotCPT5A
+            jItPOlFKbZJOjZZ5S4WfW1IZXol7daZ/joNSEm50f69od6J6Yql+dZyCoLwxtFNfipObPKuYQsPDh0uZUMTLbb2gyyy2uamAKleup3bdb1VZBeMFAqjX3VHk
+            4wbtBqvSoe7tB/UHwprKA/b1CP4I+Jzp++cvgcEEASs9G3DQoIKHClZInNgD4sMFGDHGA5URI4OPIyBDihxJsmSDkyhTqlzJsqWDlzBjypxJs+aDmzhz6tzJ
+            s2cEADs=
         }
     }
 
     proc options {this} {
         # data index must be forced so that initialization always occur
         return [list\
-            [list -columnwidths columnWidths ColumnWidths {} {}]\
+            [list -columnwidths {} {}]\
             [list -dataindex {}]\
             [list -deletecommand {} {}]\
-            [list -draggable draggable Draggable 0 0]\
-            [list -titlefont titleFont TitleFont $font::(mediumBold) $font::(mediumBold)]\
+            [list -draggable 0 0]\
+            [list -titlefont $font::(mediumBold) $font::(mediumBold)]\
         ]
     }
 
@@ -96,10 +97,10 @@ class summaryTable {
             if {$value<$(nextDataIndex)} {
                 error "specified data index ($value) is lower than internal summary table index"
             }
-            set ::summaryTable::(nextDataIndex) $value
+            set (nextDataIndex) $value
         }
         set ($this,dataName) $(nextDataIndex)data                                             ;# generate unique name based on index
-        incr ::summaryTable::(nextDataIndex)
+        incr (nextDataIndex)
     }
 
     proc set-deletecommand {this value} {}
@@ -120,21 +121,28 @@ class summaryTable {
         variable [set dataName $($this,dataName)]
         variable ${this}cellRow
 
-        viewer::registerTrace $this $array
         set cell ${array}($row,$column)
         if {[info exists ${this}cellRow($cell)]} return                                                  ;# already displayed, abort
-
+        viewer::registerTrace $this $array
         set label [viewer::label $array $row $column]
-
         set row $($this,nextRow)                                                                     ;# next row for this data table
         set ${dataName}($row,0) $label
-        # initialize average, minimum and maximum
-        array set $dataName [list $row,2 $(void) $row,3 $(void) $row,4 $(void)]
+        if {[catch {set $cell} current]} {                                                                    ;# cell does not exist
+            set ${dataName}($row,1) ?
+        } else {
+            set ${dataName}($row,1) $current
+        }
+        array set $dataName [list $row,2 ? $row,3 ? $row,4 ?]                             ;# initialize average, minimum and maximum
+        set ${dataName}($row,updates) 0
         set ${dataName}($row,sum) 0.0
         set ${this}cellRow($cell) $row                                                                          ;# remember cell row
-        # setup action when a row is deleted through a cell drop in trash
+        # setup action when a row is deleted through a cell drop in eraser site
         set ($this,rowLastWish,$row) [new lastWish "summaryTable::deleteRow $this $cell"]
         incr ($this,nextRow)
+        if {[string first ? $label]>=0} {                                                          ;# label cannot be determined yet
+            set ($this,relabel,$row) {}
+        }
+        set ${dataName}(updates) 0                      ;# let data table update itself (so colors can be set on cells, for example)
     }
 
     proc update {this array args} {
@@ -142,24 +150,30 @@ class summaryTable {
         variable ${this}cellRow
 
         foreach {cell row} [array get ${this}cellRow] {
-            if {[catch {set $cell} current]||[string equal $current $(void)]} {
-                # cell does not exist or is another summary table void cell, so propagate here
-                set ${dataName}($row,1) $(void)                         ;# do not touch other columns as their content remains valid
+            if {[string first $array $cell]<0} continue                ;# no need to update if cell does not belong to updated array
+            if {[catch {set $cell} current]||[string equal $current ?]} {                          ;# cell does not exist or is void
+                set ${dataName}($row,1) ?                               ;# do not touch other columns as their content remains valid
             } else {                                                                                                ;# data is valid
+                if {[info exists ($this,relabel,$row)]} {                                  ;# if label is not yet defined, update it
+                    viewer::parse $cell array cellRow cellColumn type
+                    set label [viewer::label $array $cellRow $cellColumn]
+                    set ${dataName}($row,0) $label
+                    if {[string first ? $label]<0} {                                                 ;# label now completely defined
+                        unset ($this,relabel,$row)
+                    }
+                }
                 set ${dataName}($row,1) $current
-                set ${dataName}($row,2) [format %.2f\
-                    [expr\
-                        {[set ${dataName}($row,sum) [expr {[set ${dataName}($row,sum)]+$current}]]/([set ${dataName}(updates)]+1)}\
-                    ]\
-                ]
+                set sum [expr {[set ${dataName}($row,sum)]+$current}]
+                set ${dataName}($row,2) [format %.2f [expr {$sum/[incr ${dataName}($row,updates)]}]]
                 set value [set ${dataName}($row,3)]
-                if {[string equal $value $(void)]||($current<$value)} {                             ;# eventually initialize minimum
+                if {[string equal $value ?]||($current<$value)} {                                   ;# eventually initialize minimum
                     set ${dataName}($row,3) $current
                 }
                 set value [set ${dataName}($row,4)]
-                if {[string equal $value $(void)]||($current>$value)} {                             ;# eventually initialize maximum
+                if {[string equal $value ?]||($current>$value)} {                                   ;# eventually initialize maximum
                     set ${dataName}($row,4) $current
                 }
+                set ${dataName}($row,sum) $sum
             }
         }
         incr ${dataName}(updates)                                                                    ;# let data table update itself
@@ -174,18 +188,37 @@ class summaryTable {
     proc dragData {this format} {
         variable ${this}cellRow
 
-        foreach cell [dataTable::dragData $($this,dataTable) $format] {                 ;# gather rows with at least 1 selected cell
-            regexp {\(([^,]+)} $cell dummy row
-            set selected($row) {}
-        }
-        set lastWishes {}
-        foreach row [array names selected] {
-            lappend lastWishes $($this,rowLastWish,$row)
-        }
-        if {[llength $lastWishes]==0} {
-            return $this                                                                          ;# self destruct if no rows remain
-        } else {
-            return $lastWishes
+        switch $format {
+            OBJECTS {
+                foreach cell [dataTable::dragData $($this,dataTable) $format] {         ;# gather rows with at least 1 selected cell
+                    regexp {\(([^,]+)} $cell dummy row
+                    set selected($row) {}
+                }
+                set lastWishes {}
+                foreach row [array names selected] {
+                    lappend lastWishes $($this,rowLastWish,$row)
+                }
+                if {[llength $lastWishes]==0} {
+                    return $this                                                                  ;# self destruct if no rows remain
+                } else {
+                    return $lastWishes
+                }
+            }
+            DATACELLS {
+                foreach {cell row} [array get ${this}cellRow] {                                ;# revert original cell / row mapping
+                    set original($row) $cell
+                }
+                set cells {}
+                foreach cell [dataTable::dragData $($this,dataTable) DATACELLS] {
+                    viewer::parse $cell array row column type
+                    if {$column==1} {                   ;# current value: replace with original cell to propagate color, for example
+                        lappend cells $original($row)
+                    } else {
+                        lappend cells $cell
+                    }
+                }
+                return $cells
+            }
         }
     }
 
@@ -195,7 +228,7 @@ class summaryTable {
 
         set row [set ${this}cellRow($cell)]
         unset ${dataName}($row,0) ${dataName}($row,1) ${dataName}($row,2) ${dataName}($row,3) ${dataName}($row,4)\
-            ${dataName}($row,sum) ($this,rowLastWish,$row)
+            ${dataName}($row,updates) ${dataName}($row,sum) ($this,rowLastWish,$row)
         unset ${this}cellRow($cell)
         dataTable::update $($this,dataTable)
     }
@@ -212,4 +245,18 @@ class summaryTable {
         }
         return $list
     }
+
+    proc setCellColor {this array row column color} {                                                   ;# cell is row current value
+        variable [set dataName $($this,dataName)]
+        variable ${this}cellRow
+
+        set source ${array}($row,$column)
+        foreach {cell row} [array get ${this}cellRow] {
+            if {[string equal $cell $source]} {
+                dataTable::setCellColor $($this,dataTable) $row 1 $color
+                return
+            }
+        }
+    }
+
 }
