@@ -1,15 +1,16 @@
-# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
+# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: viewer.tcl,v 2.1 1999/08/28 09:32:13 jfontain Exp $}
+set rcsId {$Id: viewer.tcl,v 2.9 2001/01/12 21:37:24 jfontain Exp $}
+
 
 class viewer {                                                                             ;# handle viewers related functionalities
     # viewers do not derive from a common interface class but rather support a common set a options through the composite class
 
-    set ::viewer::(list) {}
+    set (list) {}
 
     proc viewer {this} {
-        lappend ::viewer::(list) $this
+        lappend (list) $this
     }
 
     proc ~viewer {this} {
@@ -22,15 +23,17 @@ class viewer {
         if {[info exists ($this,drop)]} {
             delete $($this,drop)
         }
-        ldelete ::viewer::(list) $this
+        ldelete (list) $this
     }
 
     virtual proc supportedTypes {this}
 
     proc view {this cells} {                                                                  ;# cells is a list of data array cells
         foreach cell [parseCells $this $cells] {
-            eval monitorCell $this $cell
-            set arrays([lindex $cell 0]) {}                                                                        ;# remember array
+            foreach {array row column} $cell {}
+            monitorCell $this $array $row $column
+            setCellColor $this $array $row $column [thresholds::cellColor $array $row $column]   ;# eventually initialize cell color
+            set arrays($array) {}                                                                                  ;# remember array
         }
         foreach array [array names arrays] {                                        ;# update viewer with current values immediately
             update $this $array
@@ -54,7 +57,7 @@ class viewer {
         return $parsed
     }
 
-    proc parse {dataCell arrayName rowName columnName typeName} {
+    proc parse {dataCell arrayName rowName columnName typeName} {                                                ;# public procedure
         upvar $arrayName array $rowName row $columnName column $typeName type
 
         if {([scan $dataCell {%[^(](%u,%u)} array row column]!=3)||($row<0)||($column<0)} {
@@ -161,4 +164,12 @@ class viewer {
         manageViewer $viewer 1 -static $global::static -setx $x -sety $y -setwidth $width -setheight $height -level $level
     }
 
+    proc changeAllCellsColor {array row column color} {
+       foreach viewer $(list) {
+           setCellColor $viewer $array $row $column $color
+       }
+    }
+
+    virtual proc setCellColor {this array row column color}
+
 }
